"""
ua custom locale file.
"""
from __future__ import annotations


translations = {
    "units": {"few_second": "кілька секунд"},
    # Relative time
    "ago": "{} тому",
    "from_now": "за {}",
    "after": "{0} посіля",
    "before": "{0} до",
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "L": "DD.MM.YYYY",
        "LL": "D MMMM YYYY р.",
        "LLL": "D MMMM YYYY р., HH:mm",
        "LLLL": "dddd, D MMMM YYYY р., HH:mm",
    },
}
