"""
ja custom locale file.
"""
from __future__ import annotations


translations = {
    "units": {"few_second": "数秒"},
    # Relative time
    "ago": "{} 前に",
    "from_now": "今から {}",
    "after": "{0} 後",
    "before": "{0} 前",
    # Date formats
    "date_formats": {
        "LTS": "h:mm:ss A",
        "LT": "h:mm A",
        "L": "MM/DD/YYYY",
        "LL": "MMMM D, YYYY",
        "LLL": "MMMM D, YYYY h:mm A",
        "LLLL": "dddd, MMMM D, YYYY h:mm A",
    },
}
