"""
cs custom locale file.
"""
from __future__ import annotations


translations = {
    "units": {"few_second": "pár vteřin"},
    # Relative time
    "ago": "{} zpět",
    "from_now": "za {}",
    "after": "{0} po",
    "before": "{0} zpět",
    # Ordinals
    "ordinal": {"one": ".", "two": ".", "few": ".", "other": "."},
    # Date formats
    "date_formats": {
        "LTS": "h:mm:ss",
        "LT": "h:mm",
        "L": "DD. M. YYYY",
        "LL": "D. MMMM, YYYY",
        "LLL": "D. MMMM, YYYY h:mm",
        "LLLL": "dddd, D. MMMM, YYYY h:mm",
    },
}
