"""
sv custom locale file.
"""
from __future__ import annotations


translations = {
    # Relative time
    "ago": "{} sedan",
    "from_now": "från nu {}",
    "after": "{0} efter",
    "before": "{0} innan",
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "L": "YYYY-MM-DD",
        "LL": "D MMMM YYYY",
        "LLL": "D MMMM YYYY, HH:mm",
        "LLLL": "dddd, D MMMM YYYY, HH:mm",
    },
}
