"""
bg custom locale file.
"""
from __future__ import annotations


translations = {
    # Relative time
    "ago": "преди {}",
    "from_now": "след {}",
    "after": "след {0}",
    "before": "преди {0}",
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "L": "DD.MM.YYYY",
        "LL": "D MMMM YYYY г.",
        "LLL": "D MMMM YYYY г., HH:mm",
        "LLLL": "dddd, D MMMM YYYY г., HH:mm",
    },
}
